@@ -1,25 +1,37 @@
-# Copyrights 2023 by [Mark Overmeer <markov@cpan.org>].
-#  For other contributors see ChangeLog.
-# See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.03.
-package Math::Formula::Config::JSON;
-use vars '$VERSION';
-$VERSION = '0.16';
+# This code is part of Perl distribution Math-Formula version 0.17.
+# The POD got stripped from this file by OODoc version 3.03.
+# For contributors see file ChangeLog.
 
-use base 'Math::Formula::Config';
+# This software is copyright (c) 2023-2025 by Mark Overmeer.
+
+# This is free software; you can redistribute it and/or modify it under
+# the same terms as the Perl 5 programming language system itself.
+# SPDX-License-Identifier: Artistic-1.0-Perl OR GPL-1.0-or-later
+
+#oodist: *** DO NOT USE THIS VERSION FOR PRODUCTION ***
+#oodist: This file contains OODoc-style documentation which will get stripped
+#oodist: during its release in the distribution.  You can use this file for
+#oodist: testing, however the code of this development version may be broken!
+
+package Math::Formula::Config::JSON;{
+our $VERSION = '0.17';
+}
+
+use parent 'Math::Formula::Config';
 
 use warnings;
 use strict;
 
-use Log::Report 'math-formula';
-use Scalar::Util  'blessed';
-use File::Slurper 'read_binary';
+use Log::Report       qw/math-formula/;
+use Scalar::Util      qw/blessed/;
+use File::Slurper     qw/read_binary/;
 use Cpanel::JSON::XS  ();
 
 my $json = Cpanel::JSON::XS->new->pretty->utf8->canonical(1);
 
+#--------------------
 
-#----------------------
+#--------------------
 
 sub save($%)
 {	my ($self, $context, %args) = @_;
@@ -31,11 +43,11 @@ sub save($%)
 
 	my $fn = $self->path_for($args{filename} || "$name.json");
 	open my $fh, '>:raw', $fn
-		or fault __x"Trying to save context '{name}' to {fn}", name => $name, fn => $fn;
+		or fault __x"Trying to save context '{name}' to {file}", name => $name, file => $fn;
 
 	$fh->print($json->encode($tree));
 	$fh->close
-		or fault __x"Error on close while saving '{name}' to {fn}", name => $name, fn => $fn;
+		or fault __x"Error on close while saving '{name}' to {file}", name => $name, file => $fn;
 }
 
 sub _set($)
@@ -86,9 +98,8 @@ sub load($%)
 	my $tree     = $json->decode(read_binary $fn);
 	my $formulas = delete $tree->{formulas};
 
-	my $attrs = $self->_set_decode($tree);
-	Math::Formula::Context->new(name => $name,
-		%$attrs,
+	my $attrs    = $self->_set_decode($tree);
+	Math::Formula::Context->new(name => $name, %$attrs,
 		formulas => $self->_set_decode($formulas),
 	);
 }
@@ -125,6 +136,6 @@ sub _unpack($$)
 	: MF::STRING->new(undef, $encoded);
 }
 
-#----------------------
+#--------------------
 
 1;
