@@ -1,13 +1,24 @@
-# Copyrights 2023 by [Mark Overmeer <markov@cpan.org>].
-#  For other contributors see ChangeLog.
-# See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.03.
+# This code is part of Perl distribution Math-Formula version 0.17.
+# The POD got stripped from this file by OODoc version 3.03.
+# For contributors see file ChangeLog.
+
+# This software is copyright (c) 2023-2025 by Mark Overmeer.
+
+# This is free software; you can redistribute it and/or modify it under
+# the same terms as the Perl 5 programming language system itself.
+# SPDX-License-Identifier: Artistic-1.0-Perl OR GPL-1.0-or-later
+
+#oodist: *** DO NOT USE THIS VERSION FOR PRODUCTION ***
+#oodist: This file contains OODoc-style documentation which will get stripped
+#oodist: during its release in the distribution.  You can use this file for
+#oodist: testing, however the code of this development version may be broken!
+
 use warnings;
 use strict;
 
-package Math::Formula::Token;
-use vars '$VERSION';
-$VERSION = '0.16';
+package Math::Formula::Token;{
+our $VERSION = '0.17';
+}
 
 
 #!!! The declarations of all other packages in this file are indented to avoid
@@ -26,18 +37,16 @@ sub new(%) { my $class = shift; bless [@_], $class }
 sub token  { $_[0][0] //= $_[0]->_token($_[0]->value) }
 sub _token { $_[1] }
 
-#-------------------
 # MF::PARENS, parenthesis tokens
 # Parser object to administer parenthesis, but disappears in the AST.
 
 package
 	MF::PARENS;
 
-use base 'Math::Formula::Token';
+use base 'Math::Formula::Token';  #XXX module does not load with 'use parent'
 
 sub level { $_[0][1] }
 
-#-------------------
 # MF::OPERATOR, operator of yet unknown type.
 # In the AST upgraded to either MF::PREFIX or MF::INFIX.
 
@@ -48,8 +57,8 @@ use base 'Math::Formula::Token';
 use Log::Report 'math-formula', import => [ 'panic' ];
 
 use constant {
-    # Associativity
-    LTR => 1, RTL => 2, NOCHAIN => 3,
+	# Associativity
+	LTR => 1, RTL => 2, NOCHAIN => 3,
 };
 
 # method operator(): Returns the operator value in this token, which
@@ -64,10 +73,10 @@ sub compute
 my %table;
 {
 	# Prefix operators and parenthesis are not needed here
-    # Keep in sync with the table in Math::Formula
-	my @order = (
+	# Keep in sync with the table in Math::Formula
+	my @order =	(
 #		[ LTR,     ',' ],
- 		[ LTR,     '?', ':' ],        # ternary ?:
+		[ LTR,     '?', ':' ],        # ternary ?:
 		[ NOCHAIN, '->' ],
 		[ LTR,     qw/or xor/, '//' ],
 		[ LTR,     'and' ],
@@ -89,15 +98,14 @@ my %table;
 
 # method find($operator)
 # Returns a list with knowledge about a know operator.
-#   The first argument is a priority level for this operator.  The actual
+# The first argument is a priority level for this operator.  The actual
 # priority numbers may change over releases of this module.
-#   The second value is a constant of associativety.  Either the constant
+# The second value is a constant of associativety.  Either the constant
 # LTR (compute left to right), RTL (right to left), or NOCHAIN (non-stackable
 # operator).
 
 sub find($) { @{$table{$_[1]} // panic "op $_[1]" } }
 
-#-------------------
 # MF::PREFIX, monadic prefix operator
 # Prefix operators process the result of the expression which follows it.
 # This is a specialization from the MF::OPERATOR type, hence shares its methods.
@@ -112,13 +120,12 @@ sub child() { $_[0][1] }
 
 sub compute($$)
 {	my ($self, $context) = @_;
-    my $value = $self->child->compute($context)
+	my $value = $self->child->compute($context)
 		or return undef;
 
 	$value->prefix($self->operator, $context);
 }
 
-#-------------------
 # MF::INFIX, infix (dyadic) operator
 # Infix operators have two arguments.  This is a specialization from the
 # MF::OPERATOR type, hence shares its methods.
@@ -154,7 +161,7 @@ sub _compare_ops { keys %comparison }
 sub compute($$)
 {	my ($self, $context) = @_;
 
-    my $left  = $self->left->compute($context)
+	my $left  = $self->left->compute($context)
 		or return undef;
 
 	my $right = $self->right->compute($context)
@@ -175,7 +182,6 @@ sub compute($$)
 }
 
 
-#-------------------
 # MF::TERNARY,  if ? then : else
 # Ternary operators have three arguments.  This is a specialization from the
 # MF::OPERATOR type, hence shares its methods.
@@ -192,13 +198,12 @@ sub else()      { $_[0][3] }
 sub compute($$)
 {	my ($self, $context) = @_;
 
-    my $cond  = $self->condition->compute($context)
+	my $cond  = $self->condition->compute($context)
 		or return undef;
 
 	($cond->value ? $self->then : $self->else)->compute($context)
 }
 
-#-------------------
 # When used, this returns a MF::STRING taken from the captures in the context.
 
 package
